@@ -11,40 +11,15 @@ import datetime  # NOQA
 import decimal  # NOQA
 import orderly_set  # NOQA
 import collections  # NOQA
-try:
-    import yaml
-except ImportError:  # pragma: no cover.
-    yaml = None  # pragma: no cover.
-try:
-    if sys.version_info >= (3, 11):
-        import tomllib as tomli
-    else:
-        import tomli
-except ImportError:  # pragma: no cover.
-    tomli = None  # pragma: no cover.
-try:
-    import tomli_w
-except ImportError:  # pragma: no cover.
-    tomli_w = None  # pragma: no cover.
-try:
-    import clevercsv
-    csv = None
-except ImportError:  # pragma: no cover.
-    import csv
-    clevercsv = None  # pragma: no cover.
-try:
-    import orjson
-except ImportError:  # pragma: no cover.
-    orjson = None
-try:
-    from pydantic import BaseModel as PydanticBaseModel
-except ImportError:  # pragma: no cover.
-    PydanticBaseModel = None
-
+import ipaddress
+import base64
 from copy import deepcopy, copy
 from functools import partial
-from collections.abc import Mapping
-from typing import Callable, Optional, Union
+from collections.abc import Mapping, KeysView
+from typing import (
+    Callable, Optional, Union,
+    overload, Literal, Any,
+)
 from deepdiff.helper import (
     strings,
     get_type,
@@ -56,16 +31,19 @@ from deepdiff.helper import (
     np_ndarray,
     Opcode,
     SetOrdered,
+    pydantic_base_model_type,
+    PydanticBaseModel,
+    NotPresent,
+    ipranges,
 )
 from deepdiff.model import DeltaResult
 
-logger = logging.getLogger(__name__)
-
 try:
-    import jsonpickle
-except ImportError:  # pragma: no cover. Json pickle is getting deprecated.
-    jsonpickle = None  # pragma: no cover. Json pickle is getting deprecated.
+    import orjson
+except ImportError:  # pragma: no cover.
+    orjson = None
 
+logger = logging.getLogger(__name__)
 
 class UnsupportedFormatErr(TypeError):
     pass
@@ -110,6 +88,13 @@ SAFE_TO_IMPORT = {
     'collections.OrderedDict',
     're.Pattern',
     'deepdiff.helper.Opcode',
+    'ipaddress.IPv4Interface',
+    'ipaddress.IPv6Interface',
+    'ipaddress.IPv4Network',
+    'ipaddress.IPv6Network',
+    'ipaddress.IPv4Address',
+    'ipaddress.IPv6Address',
+    'collections.abc.KeysView',
 }
 
 
@@ -137,7 +122,11 @@ TYPE_STR_TO_TYPE = {
     'SetOrdered': SetOrdered,
     'namedtuple': collections.namedtuple,
     'OrderedDict': collections.OrderedDict,
-    'Pattern': re.Pattern,    
+    'Pattern': re.Pattern,
+    'iprange': str,
+    'IPv4Address': ipaddress.IPv4Address,
+    'IPv6Address': ipaddress.IPv6Address,
+    'KeysView': list,
 }
 
 
@@ -162,10 +151,11 @@ class SerializationMixin:
         :ref:`to_json_pickle_label`
         Get the json pickle of the diff object. Unless you need all the attributes and functionality of DeepDiff, running to_json() is the safer option that json pickle.
         """
-        if jsonpickle:
-            copied = self.copy()
+        try:
+            import jsonpickle
+            copied = self.copy()  # type: ignore
             return jsonpickle.encode(copied)
-        else:
+        except ImportError:  # pragma: no cover. Json pickle is getting deprecated.
             logger.error('jsonpickle library needs to be installed in order to run to_json_pickle')  # pragma: no cover. Json pickle is getting deprecated.
 
     @classmethod
@@ -175,9 +165,10 @@ class SerializationMixin:
         Load DeepDiff object with all the bells and whistles from the json pickle dump.
         Note that json pickle dump comes from to_json_pickle
         """
-        if jsonpickle:
+        try:
+            import jsonpickle
             return jsonpickle.decode(value)
-        else:
+        except ImportError:  # pragma: no cover. Json pickle is getting deprecated.
             logger.error('jsonpickle library needs to be installed in order to run from_json_pickle')  # pragma: no cover. Json pickle is getting deprecated.
 
     def to_json(self, default_mapping: Optional[dict]=None, force_use_builtin_json=False, **kwargs):
@@ -225,7 +216,7 @@ class SerializationMixin:
             **kwargs,
         )
 
-    def to_dict(self, view_override=None):
+    def to_dict(self, view_override: Optional[str]=None) -> dict:
         """
         convert the result to a python dictionary. You can override the view type by passing view_override.
 
@@ -236,10 +227,15 @@ class SerializationMixin:
             The options are the text or tree.
         """
 
-        view = view_override if view_override else self.view
-        return dict(self._get_view_results(view))
+        view = view_override if view_override else self.view  # type: ignore
+        return dict(self._get_view_results(view))  # type: ignore
 
-    def _to_delta_dict(self, directed=True, report_repetition_required=True, always_include_values=False):
+    def _to_delta_dict(
+        self,
+        directed: bool = True,
+        report_repetition_required: bool = True,
+        always_include_values: bool = False,
+    ) -> dict:
         """
         Dump to a dictionary suitable for delta usage.
         Unlike to_dict, this is not dependent on the original view that the user chose to create the diff.
@@ -262,12 +258,12 @@ class SerializationMixin:
         was set to be True in the diff object.
 
         """
-        if self.group_by is not None:
+        if self.group_by is not None:  # type: ignore
             raise ValueError(DELTA_ERROR_WHEN_GROUP_BY)
 
         if directed and not always_include_values:
-            _iterable_opcodes = {}
-            for path, op_codes in self._iterable_opcodes.items():
+            _iterable_opcodes = {}  # type: ignore
+            for path, op_codes in self._iterable_opcodes.items():  # type: ignore
                 _iterable_opcodes[path] = []
                 for op_code in op_codes:
                     new_op_code = Opcode(
@@ -280,29 +276,29 @@ class SerializationMixin:
                     )
                     _iterable_opcodes[path].append(new_op_code)
         else:
-            _iterable_opcodes = self._iterable_opcodes
+            _iterable_opcodes = self._iterable_opcodes  # type: ignore
 
         result = DeltaResult(
-            tree_results=self.tree,
-            ignore_order=self.ignore_order,
+            tree_results=self.tree,  # type: ignore
+            ignore_order=self.ignore_order,  # type: ignore
             always_include_values=always_include_values,
             _iterable_opcodes=_iterable_opcodes,
         )
         result.remove_empty_keys()
-        if report_repetition_required and self.ignore_order and not self.report_repetition:
+        if report_repetition_required and self.ignore_order and not self.report_repetition:  # type: ignore
             raise ValueError(DELTA_IGNORE_ORDER_NEEDS_REPETITION_REPORT)
         if directed:
             for report_key, report_value in result.items():
                 if isinstance(report_value, Mapping):
                     for path, value in report_value.items():
                         if isinstance(value, Mapping) and 'old_value' in value:
-                            del value['old_value']
-        if self._numpy_paths:
+                            del value['old_value']  # type: ignore
+        if self._numpy_paths:  # type: ignore
             # Note that keys that start with '_' are considered internal to DeepDiff
             # and will be omitted when counting distance. (Look inside the distance module.)
-            result['_numpy_paths'] = self._numpy_paths
+            result['_numpy_paths'] = self._numpy_paths  # type: ignore
 
-        if self.iterable_compare_func:
+        if self.iterable_compare_func:  # type: ignore
             result['_iterable_compare_func_was_used'] = True
 
         return deepcopy(dict(result))
@@ -325,9 +321,9 @@ class SerializationMixin:
         result = []
         if prefix is None:
             prefix = ''
-        keys = sorted(self.tree.keys())  # sorting keys to guarantee constant order across python versions.
+        keys = sorted(self.tree.keys())  # type: ignore # sorting keys to guarantee constant order across python versions.
         for key in keys:
-            for item_key in self.tree[key]:
+            for item_key in self.tree[key]:  # type: ignore
                 result += [pretty_print_diff(item_key)]
 
         if callable(prefix):
@@ -363,8 +359,8 @@ class _RestrictedUnpickler(pickle.Unpickler):
         # Forbid everything else.
         raise ForbiddenModule(FORBIDDEN_MODULE_MSG.format(module_dot_class)) from None
 
-    def persistent_load(self, persistent_id):
-        if persistent_id == "<<NoneType>>":
+    def persistent_load(self, pid):
+        if pid == "<<NoneType>>":
             return type(None)
 
 
@@ -483,19 +479,27 @@ def load_path_content(path, file_type=None):
     """
     Loads and deserializes the content of the path.
     """
+
     if file_type is None:
         file_type = path.split('.')[-1]
     if file_type == 'json':
         with open(path, 'r') as the_file:
             content = json_loads(the_file.read())
     elif file_type in {'yaml', 'yml'}:
-        if yaml is None:  # pragma: no cover.
-            raise ImportError('Pyyaml needs to be installed.')  # pragma: no cover.
+        try:
+            import yaml
+        except ImportError:  # pragma: no cover.
+            raise ImportError('Pyyaml needs to be installed.') from None  # pragma: no cover.
         with open(path, 'r') as the_file:
             content = yaml.safe_load(the_file)
     elif file_type == 'toml':
-        if tomli is None:  # pragma: no cover.
-            raise ImportError('On python<=3.10 tomli needs to be installed.')  # pragma: no cover.
+        try:
+            if sys.version_info >= (3, 11):
+                import tomllib as tomli
+            else:
+                import tomli
+        except ImportError:  # pragma: no cover.
+            raise ImportError('On python<=3.10 tomli needs to be installed.') from None  # pragma: no cover.
         with open(path, 'rb') as the_file:
             content = tomli.load(the_file)
     elif file_type == 'pickle':
@@ -503,11 +507,14 @@ def load_path_content(path, file_type=None):
             content = the_file.read()
             content = pickle_load(content)
     elif file_type in {'csv', 'tsv'}:
-        if clevercsv:  # pragma: no cover.
+        try:
+            import clevercsv  # type: ignore
             content = clevercsv.read_dicts(path)
-        else:
+        except ImportError:  # pragma: no cover.
+            import csv
             with open(path, 'r') as the_file:
                 content = list(csv.DictReader(the_file))
+
         logger.info(f"NOTE: CSV content was empty in {path}")
 
         # Everything in csv is string but we try to automatically convert any numbers we find
@@ -552,24 +559,30 @@ def _save_content(content, path, file_type, keep_backup=True):
     if file_type == 'json':
         with open(path, 'w') as the_file:
             content = json_dumps(content)
-            the_file.write(content)
+            the_file.write(content)  # type: ignore
     elif file_type in {'yaml', 'yml'}:
-        if yaml is None:  # pragma: no cover.
-            raise ImportError('Pyyaml needs to be installed.')  # pragma: no cover.
+        try:
+            import yaml
+        except ImportError:  # pragma: no cover.
+            raise ImportError('Pyyaml needs to be installed.') from None  # pragma: no cover.
         with open(path, 'w') as the_file:
             content = yaml.safe_dump(content, stream=the_file)
     elif file_type == 'toml':
-        if tomli_w is None:  # pragma: no cover.
-            raise ImportError('Tomli-w needs to be installed.')  # pragma: no cover.
+        try:
+            import tomli_w
+        except ImportError:  # pragma: no cover.
+            raise ImportError('Tomli-w needs to be installed.') from None  # pragma: no cover.
         with open(path, 'wb') as the_file:
             content = tomli_w.dump(content, the_file)
     elif file_type == 'pickle':
         with open(path, 'wb') as the_file:
             content = pickle_dump(content, file_obj=the_file)
     elif file_type in {'csv', 'tsv'}:
-        if clevercsv:  # pragma: no cover.
+        try:
+            import clevercsv  # type: ignore
             dict_writer = clevercsv.DictWriter
-        else:
+        except ImportError:  # pragma: no cover.
+            import csv
             dict_writer = csv.DictWriter
         with open(path, 'w', newline='') as csvfile:
             fieldnames = list(content[0].keys())
@@ -595,13 +608,25 @@ def _serialize_tuple(value):
     return value
 
 
+def _serialize_bytes(value):
+    """
+    Serialize bytes to JSON-compatible format.
+    First tries UTF-8 decoding for backward compatibility.
+    Falls back to base64 encoding for binary data.
+    """
+    try:
+        return value.decode('utf-8')
+    except UnicodeDecodeError:
+        return base64.b64encode(value).decode('ascii')
+
+
 JSON_CONVERTOR = {
     decimal.Decimal: _serialize_decimal,
     SetOrdered: list,
     orderly_set.StableSetEq: list,
     set: list,
     type: lambda x: x.__name__,
-    bytes: lambda x: x.decode('utf-8'),
+    bytes: _serialize_bytes,
     datetime.datetime: lambda x: x.isoformat(),
     uuid.UUID: lambda x: str(x),
     np_float32: float,
@@ -611,10 +636,14 @@ JSON_CONVERTOR = {
     np_ndarray: lambda x: x.tolist(),
     tuple: _serialize_tuple,
     Mapping: dict,
+    NotPresent: str,
+    ipranges: str,
+    memoryview: lambda x: x.tobytes(),
+    KeysView: list,
 }
 
-if PydanticBaseModel:
-    JSON_CONVERTOR[PydanticBaseModel] = lambda x: x.dict()
+if PydanticBaseModel is not pydantic_base_model_type:
+    JSON_CONVERTOR[PydanticBaseModel] = lambda x: x.model_dump()
 
 
 def json_convertor_default(default_mapping=None):
@@ -631,7 +660,33 @@ def json_convertor_default(default_mapping=None):
         # This is to handle reverse() which creates a generator of type list_reverseiterator
         if obj.__class__.__name__ == 'list_reverseiterator':
             return list(copy(obj))
-        raise TypeError('We do not know how to convert {} of type {} for json serialization. Please pass the default_mapping parameter with proper mapping of the object to a basic python type.'.format(obj, type(obj)))
+        # 3) gather @property values by scanning __class__.__dict__ and bases
+        props = {}
+        for cls in obj.__class__.__mro__:
+            for name, descriptor in cls.__dict__.items():
+                if isinstance(descriptor, property) and not name.startswith('_'):
+                    try:
+                        props[name] = getattr(obj, name)
+                    except Exception:
+                        # skip properties that error out
+                        pass
+        if props:
+            return props
+
+        # 4) fallback: public __dict__ entries
+        if hasattr(obj, '__dict__'):
+            return {
+                k: v
+                for k, v in vars(obj).items()
+                if not k.startswith('_')
+            }
+
+        # 5) give up
+        raise TypeError(
+            f"Don't know how to JSON-serialize {obj!r} "
+            f"(type {type(obj).__name__}); "
+            "consider adding it to default_mapping."
+        )
 
     return _convertor
 
@@ -641,7 +696,7 @@ class JSONDecoder(json.JSONDecoder):
     def __init__(self, *args, **kwargs):
         json.JSONDecoder.__init__(self, object_hook=self.object_hook, *args, **kwargs)
 
-    def object_hook(self, obj):
+    def object_hook(self, obj):  # type: ignore
         if 'old_type' in obj and 'new_type' in obj:
             for type_key in ('old_type', 'new_type'):
                 type_str = obj[type_key]
@@ -650,7 +705,44 @@ class JSONDecoder(json.JSONDecoder):
         return obj
 
 
-def json_dumps(item, default_mapping=None, force_use_builtin_json: bool=False, **kwargs):
+
+@overload
+def json_dumps(
+    item: Any,
+    **kwargs,
+) -> str:
+    ...
+
+
+@overload
+def json_dumps(
+    item: Any,
+    default_mapping:Optional[dict],
+    force_use_builtin_json: bool,
+    return_bytes:Literal[True],
+    **kwargs,
+) -> bytes:
+    ...
+
+
+@overload
+def json_dumps(
+    item: Any,
+    default_mapping:Optional[dict],
+    force_use_builtin_json: bool,
+    return_bytes:Literal[False],
+    **kwargs,
+) -> str:
+    ...
+
+
+def json_dumps(
+    item: Any,
+    default_mapping:Optional[dict]=None,
+    force_use_builtin_json: bool = False,
+    return_bytes: bool = False,
+    **kwargs,
+) -> Union[str, bytes]:
     """
     Dump json with extra details that are not normally json serializable
 
@@ -663,22 +755,29 @@ def json_dumps(item, default_mapping=None, force_use_builtin_json: bool=False, *
     """
     if orjson and not force_use_builtin_json:
         indent = kwargs.pop('indent', None)
+        kwargs['option'] = orjson.OPT_NON_STR_KEYS | orjson.OPT_SERIALIZE_NUMPY
         if indent:
-            kwargs['option'] = orjson.OPT_INDENT_2
+            kwargs['option'] |= orjson.OPT_INDENT_2
         if 'sort_keys' in kwargs:
             raise TypeError(
                 "orjson does not accept the sort_keys parameter. "
                 "If you need to pass sort_keys, set force_use_builtin_json=True "
                 "to use Python's built-in json library instead of orjson.")
-        return orjson.dumps(
-            item,
-            default=json_convertor_default(default_mapping=default_mapping),
-            **kwargs).decode(encoding='utf-8')
-    else:
-        return json.dumps(
+        result = orjson.dumps(
             item,
             default=json_convertor_default(default_mapping=default_mapping),
             **kwargs)
+        if return_bytes:
+            return result
+        return result.decode(encoding='utf-8')
+    else:
+        result = json.dumps(
+            item,
+            default=json_convertor_default(default_mapping=default_mapping),
+            **kwargs)
+        if return_bytes:
+            return result.encode(encoding='utf-8')
+        return result
 
 
 json_loads = partial(json.loads, cls=JSONDecoder)
